/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef CKMALLOC_H
#define CKMALLOC_H

/*
 * if compiled with malloc debugging, don't really use this module so
 * dmalloc will report the proper __LINE__ and __FILE__.
 */
#ifdef USE_DMALLOC
# define ckmalloc(size)		malloc(size)
# define ckcalloc(n, size)	calloc(n, size)
# define ckrealloc(ptr, size)	realloc(ptr, size)
# define ckstrdup(s)		strdup(s)
#else

/*
 * malloc wrappers that turn an out-of-virtual-memory condition into
 * a fatal error.
 *
 * XXX: it would be nice to add a 'dump' feature, so if malloc returns
 * NULL we can dump all buffers out to recovery files and allow the user
 * to get any unsaved data.
 */

static __inline void *ckmalloc(size_t size) {
	void *p;

	assert(size);
	p = malloc(size);
	if (!p)
		err(1, "malloc failure; out of virtual memory");
	return p;
}

static __inline void *ckcalloc(size_t number, size_t size) {
	void *p;

	assert(size);
	p = calloc(number, size);
	if (!p)
		err(1, "calloc failure; out of virtual memory");
	return p;
}

static __inline void *ckrealloc(void *ptr, size_t size) {
	void *p;

	assert(size);
	p = realloc(ptr, size);
	if (!p)
		err(1, "realloc failure; out of virtual memory");
	return p;
}

static __inline void *ckstrdup(char *s) {
	char *p;

	p = strdup(s);
	if (!p)
		err(1, "strdup failure; out of virtual memory");
	return p;
}

#endif
#endif
