/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

/*
 * basic editing commands; delete and backspace.
 */

/*
 * delete the character under the cursor, if on the last column
 * join the next line to this one.
 */
void command_delete(vdefault_t *v) {
	line_t *next;

	v->rm_protect = PROTECT_CURSOR | PROTECT_VIEW;
	if (v->cursor.col == v->cursor.line->length) {
		next = TAILQ_NEXT(v->cursor.line, l_list);
		if (!next) {
			screen_bell();
			return;
		}

		undo_beginblock(v->hdr.buffer);
		buffer_nputs(v->hdr.buffer, 1, v->cursor.line,
			v->cursor.linenum, v->cursor.col, next->text,
			next->length);
		buffer_rmline(v->hdr.buffer, 1, next, v->cursor.linenum + 1);
		undo_endblock(v->hdr.buffer);

		vdefault_checkscroll(v);
		v->redraw_header = 1;
	} else
		buffer_rmch(v->hdr.buffer, 1, v->cursor.line,
			v->cursor.linenum, v->cursor.col);

	v->redraw_text = 1;
}

/*
 * delete the character preceding the cursor; if the cursor is in
 * column 0 join this line to the previous line.
 */
void command_backspace(vdefault_t *v) {
	line_t *prev;
	int len;

	v->rm_protect = PROTECT_CURSOR | PROTECT_VIEW;
	if (v->cursor.col == 0) {
		prev = TAILQ_PREV(v->cursor.line, linelist, l_list);
		if (!prev) {
			screen_bell();
			return;
		}

		len = prev->length;
		undo_beginblock(v->hdr.buffer);
		buffer_nputs(v->hdr.buffer, 1, prev, v->cursor.linenum - 1,
			prev->length, v->cursor.line->text,
			v->cursor.line->length);
		if (v->view.line == v->cursor.line) {
			v->view.line = prev;
			v->view.linenum--;
		}
		buffer_rmline(v->hdr.buffer, 1, v->cursor.line,
			v->cursor.linenum);
		v->cursor.line = prev;
		v->cursor.linenum--;
		v->cursor.col = len;
		undo_endblock(v->hdr.buffer);
	} else {
		v->cursor.col--;
		buffer_rmch(v->hdr.buffer, 1, v->cursor.line,
			v->cursor.linenum, v->cursor.col);
	}

	vdefault_setscreencol(v);
	vdefault_checkscroll(v);
	v->redraw_header = 1;
	v->redraw_text = 1;
}
