/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

void command_insertfile(vdefault_t *v) {
	u_char *fn, *exfn;
	u_char *dir;
	FILE *fp;
	line_t *line;
	int linenum, col;

	/* get the filename */
	dir = dirpart(v->hdr.buffer->name);
	fn = minibuff_fileprompt(dir);
	if (dir)
		free(dir);
	if (!fn)
		return;
	exfn = tilde_expand(fn);
	free(fn);

	input_begincancel();

	fp = fopen(exfn, "rb");
	if (!fp) {
		minibuff_printf("Unable to open file: %s", strerror(errno));
		goto free;
	}
	if (input_checkcancel())
		goto close;

	/* read in the file */
	line = v->cursor.line;
	linenum = v->cursor.linenum;
	col = v->cursor.col;
	if (file_fread(v->hdr.buffer, 1, 1, &line, &linenum, &col, fp))
		minibuff_printf("Error while reading file: %s",
			strerror(errno));

	/* place a mark at the end of the file insertion */
	buffer_pushmark(v->hdr.buffer, line, linenum, col);
	v->hdr.buffer->mark_implicit = 1;

close:
	fclose(fp);
free:
	input_endcancel();
	free(exfn);
}
