/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

void command_setmark(vdefault_t *v) {
	buffer_pushmark(v->hdr.buffer, v->cursor.line,
		v->cursor.linenum, v->cursor.col);
	v->hdr.buffer->mark_implicit = options.noexplicitmark;
	minibuff_printf("Mark set.");
}

void command_popmark(vdefault_t *v) {
	if (!v->hdr.buffer->mark) {
		minibuff_printf("No mark to pop.");
		return;
	}

	v->cursor.line = v->hdr.buffer->mark->line;
	v->cursor.linenum = v->hdr.buffer->mark->linenum;
	v->cursor.col = v->hdr.buffer->mark->col;
	buffer_popmark(v->hdr.buffer);
	if (!v->hdr.buffer->mark_implicit)
		v->redraw_text = 1;
	v->hdr.buffer->mark_implicit = 1;
	minibuff_printf("Mark popped.");
	vdefault_setscreencol(v);
	vdefault_checkscroll(v);
	v->redraw_header = 1;
}

void command_promotemark(vdefault_t *v) {
	if (!v->hdr.buffer->mark)
		minibuff_printf("Mark isn't set.");
	else if (v->hdr.buffer->mark_implicit) {
		minibuff_printf("Implicit mark promoted to explicit mark.");
		v->hdr.buffer->mark_implicit = 0;
		v->redraw_header = 1;
	}
}

void command_demotemark(vdefault_t *v) {
	if (!v->hdr.buffer->mark)
		minibuff_printf("Mark isn't set.");
	else if (!v->hdr.buffer->mark_implicit) {
		minibuff_printf("Explicit mark demoted to implicit mark.");
		v->hdr.buffer->mark_implicit = 1;
		v->redraw_text = 1;
	}
}

void command_xchgmark(vdefault_t *v) {
	line_t *l;
	int t;

	if (!v->hdr.buffer->mark) {
		minibuff_printf("Mark isn't set.");
		return;
	}

	swap(l, v->cursor.line, v->hdr.buffer->mark->line);
	swap(t, v->cursor.linenum, v->hdr.buffer->mark->linenum);
	swap(t, v->cursor.col, v->hdr.buffer->mark->col);
	vdefault_setscreencol(v);
	vdefault_checkscroll(v);
	v->redraw_header = 1;
}
