/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

/* hash for names to command keysyms */
commandlist_t		*command_hash;

/* hash for keysyms to commands */
static commandlist_t	*command_reverse;
#define CMDREVHASH(i)	((i) % COMMAND_BUCKETS)

void command_init() {
	command_hash = hash_alloc("command_hash", COMMAND_BUCKETS,
		offsetof(command_t, c_list));
	command_reverse = hash_alloc("command_reverse", COMMAND_BUCKETS,
		offsetof(command_t, c_rev));

	/* main commands */
	command_add("quit",		KEYSYM_QUIT);
	command_add("suspend",		KEYSYM_SUSPEND);
	command_add("cancel",		KEYSYM_CANCEL);
	command_add("excmd",		KEYSYM_EXCMD);
	command_add("hashstats",	KEYSYM_HASHSTATS);
	command_add("savebuffers",	KEYSYM_SAVEBUFFERS);
	command_add("switchbuff",	KEYSYM_SWITCHBUFF);
	command_add("setopt",		KEYSYM_SETOPT);
	command_add("openfile",		KEYSYM_OPENFILE);
	command_add("save",		KEYSYM_SAVE);
	command_add("saveas",		KEYSYM_SAVEAS);
	command_add("redraw",		KEYSYM_REDRAW);
	command_add("quotenext",	KEYSYM_QUOTENEXT);
	command_add("again",		KEYSYM_AGAIN);
	command_add("viewcycle",	KEYSYM_VIEWCYCLE);
	command_add("pushview",		KEYSYM_PUSHVIEW);
	command_add("popview",		KEYSYM_POPVIEW);
	command_add("vertsplit",	KEYSYM_VERTSPLIT);
	command_add("horizsplit",	KEYSYM_HORIZSPLIT);
	command_add("rmview",		KEYSYM_RMVIEW);
	command_add("rmotherviews",	KEYSYM_RMOTHERVIEWS);
	command_add("vertgrow",		KEYSYM_VERTGROW);
	command_add("vertshrink",	KEYSYM_VERTSHRINK);
	command_add("horizgrow",	KEYSYM_HORIZGROW);
	command_add("horizshrink",	KEYSYM_HORIZSHRINK);
	command_add("framecycle",	KEYSYM_FRAMECYCLE);
	command_add("newframe",		KEYSYM_NEWFRAME);
	command_add("rmframe",		KEYSYM_RMFRAME);

	/* keyboard macro commands */
	command_add("record",		KEYSYM_RECORD);
	command_add("playback",		KEYSYM_PLAYBACK);
	command_add("repeat",		KEYSYM_REPEAT);

	/* extended editing keys supported from the vdefault layer */
	command_add("openline",		KEYSYM_OPENLINE);
	command_add("tognlstyle",	KEYSYM_TOGNLSTYLE);
	command_add("undo",		KEYSYM_UNDO);
	command_add("redo",		KEYSYM_REDO);
	command_add("nextword",		KEYSYM_NEXTWORD);
	command_add("prevword",		KEYSYM_PREVWORD);
	command_add("nextpara",		KEYSYM_NEXTPARA);
	command_add("prevpara",		KEYSYM_PREVPARA);
	command_add("viewtop",		KEYSYM_VIEWTOP);
	command_add("viewbottom",	KEYSYM_VIEWBOTTOM);
	command_add("lineup",		KEYSYM_LINEUP);
	command_add("linedn",		KEYSYM_LINEDN);
	command_add("firstline",	KEYSYM_FIRSTLINE);
	command_add("lastline",		KEYSYM_LASTLINE);
	command_add("halfpgup",		KEYSYM_HALFPGUP);
	command_add("halfpgdn",		KEYSYM_HALFPGDN);
	command_add("center",		KEYSYM_CENTER);
	command_add("tranchars",	KEYSYM_TRANCHARS);
	command_add("tranwords",	KEYSYM_TRANWORDS);
	command_add("tranlines",	KEYSYM_TRANLINES);
	command_add("setmark",		KEYSYM_SETMARK);
	command_add("popmark",		KEYSYM_POPMARK);
	command_add("promotemark",	KEYSYM_PROMOTEMARK);
	command_add("demotemark",	KEYSYM_DEMOTEMARK);
	command_add("xchgmark",		KEYSYM_XCHGMARK);
	command_add("killrgn",		KEYSYM_KILLRGN);
	command_add("killline",		KEYSYM_KILLLINE);
	command_add("killeol",		KEYSYM_KILLEOL);
	command_add("killword",		KEYSYM_KILLWORD);
	command_add("fwdkillword",	KEYSYM_FWDKILLWORD);
	command_add("appendkill",	KEYSYM_APPENDKILL);
	command_add("yank",		KEYSYM_YANK);
	command_add("yankpop",		KEYSYM_YANKPOP);
	command_add("fwdyankpop",	KEYSYM_FWDYANKPOP);
	command_add("isearch",		KEYSYM_ISEARCH);
	command_add("risearch",		KEYSYM_RISEARCH);
	command_add("gotoln",		KEYSYM_GOTOLN);
	command_add("replace",		KEYSYM_REPLACE);
	command_add("filter",		KEYSYM_FILTER);
	command_add("insertfile",	KEYSYM_INSERTFILE);

	/* simple editing keys */
	command_add("enter",		KEYSYM_ENTER);
	command_add("up",		KEYSYM_UP);
	command_add("down",		KEYSYM_DOWN);
	command_add("left",		KEYSYM_LEFT);
	command_add("right",		KEYSYM_RIGHT);
	command_add("home",		KEYSYM_HOME);
	command_add("end",		KEYSYM_END);
	command_add("pgup",		KEYSYM_PGUP);
	command_add("pgdn",		KEYSYM_PGDN);
	command_add("backspace",	KEYSYM_BACKSPACE);
	command_add("delete",		KEYSYM_DELETE);
}

void command_shutdown() {
	command_t *command;
	int i;

	/* free the command hashes */
	for (i = 0; i < COMMAND_BUCKETS; i++)
		while (!LIST_EMPTY(&command_hash[i])) {
			command = LIST_FIRST(&command_hash[i]);
			LIST_REMOVE(command, c_list);
			LIST_REMOVE(command, c_rev);
			free(command->name);
			free(command);
		}
	hash_free(command_hash);
	hash_free(command_reverse);
}

void command_add(u_char *name, int keysym) {
	command_t *command;
	int idx;

	command = ckmalloc(sizeof(command_t));
	command->type = COMMAND_NORMAL;
	command->name = ckstrdup(name);
	command->keysym = keysym;
	idx = hash_string(COMMAND_BUCKETS, name);
	LIST_INSERT_HEAD(&command_hash[idx], command, c_list);
	idx = CMDREVHASH(keysym);
	LIST_INSERT_HEAD(&command_reverse[idx], command, c_rev);
}

void command_addf(u_char *name, int keysym, void (*handler)(command_t *)) {
	command_t *command;
	int idx;

	command = ckmalloc(sizeof(command_t));
	command->type = COMMAND_FUNC;
	command->name = ckstrdup(name);
	command->keysym = keysym;
	command->handler = handler;
	idx = hash_string(COMMAND_BUCKETS, name);
	LIST_INSERT_HEAD(&command_hash[idx], command, c_list);
	idx = CMDREVHASH(keysym);
	LIST_INSERT_HEAD(&command_reverse[idx], command, c_rev);
}

void command_remove(u_char *name) {
	command_t *command;
	int idx;

	idx = hash_string(COMMAND_BUCKETS, name);
	LIST_FOREACH(command, &command_hash[idx], c_list)
		if (!strcmp(command->name, name)) {
			LIST_REMOVE(command, c_rev);
			LIST_REMOVE(command, c_list);
			free(command->name);
			free(command);
			return;
		}

	/* don't remove commands that don't exist... */
	assert(0);
}

int command_getkeysym(u_char *name) {
	command_t *command;
	int idx;

	idx = hash_string(COMMAND_BUCKETS, name);
	LIST_FOREACH(command, &command_hash[idx], c_list)
		if (!strcmp(command->name, name))
			return command->keysym;

	return KEYSYM_NOP;
}

command_t *command_getcmd(int keysym) {
	command_t *command;
	int idx;

	idx = CMDREVHASH(keysym);
	LIST_FOREACH(command, &command_reverse[idx], c_list)
		if (command->keysym == keysym)
			return command;

	return NULL;
}

int command_excmd() {
	u_char *name;
	int keysym;

	/* make sure we aren't in the minibuff already */
	if (minibuff_editing)
		return KEYSYM_NOP;

	/* get a name of a command */
	name = minibuff_cmdprompt(NULL);
	if (!name)
		return KEYSYM_NOP;
	keysym = command_getkeysym(name);
	free(name);

	/* some keysyms aren't allowed to be run this way */
	if (keysym == KEYSYM_QUOTENEXT || keysym == KEYSYM_REDRAW ||
			keysym == KEYSYM_AGAIN) {
		minibuff_printf("Command can't be run in this manner.");
		return KEYSYM_NOP;
	}

	/* let the user know why it doesn't work */
	if (keysym == KEYSYM_NOP)
		minibuff_printf("Unknown command.");

	return keysym;
}

/* print out some hashtable statistics */
void command_hashstats() {
#ifndef NDEBUG
	hashtab_t *hashtab;
	u_char *s;
	int y;

	screen_clear();
	screen_refresh();

	y = 1;
	screen_setcolor(COLOR_DEFAULT);
	screen_gotoxy(0, 0);
	screen_puts("Hashtable stats:");
	LIST_FOREACH(hashtab, &hash_list, ht_list) {
		screen_gotoxy(0, ++y);
		screen_puts(hashtab->name);
		screen_putc(':');
		screen_gotoxy(0, ++y);
		s = hash_stats(hashtab->table, hashtab->buckets,
			hashtab->off_next);
		screen_puts(s);
		free(s);
	}
	keybind_getkey();

	screen_clear();
	frame_forcedraw(frame_active);
	minibuff_forcedraw();
#else
	minibuff_printf("This command is not available #ifdef NDEBUG.");
#endif
}

/*
 * mutli-buffer save prompting code; seperated into this routine so that
 * the various commands which close views can use it.  returns 0 to indicate
 * the loop should continue, 1 to indicate to finish without saving any more
 * buffers, and -1 to indicate that the loop was canceled.
 */
static int bufsaveloop(buffer_t *buffer, int *all, int aq) {
	u_char prompt[PATH_MAX];
	int key, error;

	if (!buffer->modified || buffer->internal)
		return 0;
	error = 0;
again:
	if (!*all) {
		if (error) {
			strlcpy(prompt, strerror(errno), sizeof(prompt));
			strlcat(prompt, ".  Save buffer ", sizeof(prompt));
		} else
			strlcpy(prompt, "Save buffer ", sizeof(prompt));
		strlcat(prompt, buffer->dispname, sizeof(prompt));
		minibuff_active = 1;
		key = minibuff_chprompt(prompt, aq ? "ynaq" : "yn", 1);
	} else
		key = 'y';

	switch (key) {
	case KEYSYM_CANCEL:
		return -1;
	case 'a':
		*all = 1;
	case 'y':
		if (buffer_save(buffer)) {
			*all = 0;
			error = 1;
			goto again;
		}
	case 'n':
		return 0;
	case 'q':
		return 1;
	}

	return 0;
}

/*
 * prompt about saving any modified buffers loaded into the editor;
 * return nonzero if the user canceled the save.
 */
int command_savebuffers() {
	buffer_t *buffer;
	int all, ret;

	all = 0;
	TAILQ_FOREACH(buffer, &buffer_list, b_list)
		if ((ret = bufsaveloop(buffer, &all, 1)) != 0)
			return ret == -1 ? -1 : 0;

	return 0;
}

void command_switchbuff() {
	buffer_t *buffer;
	u_char *buffname;

	buffname = minibuff_buffprompt(NULL);
	if (!buffname)
		return;

	buffer = buffer_lookup(buffname);
	if (!buffer) {
		minibuff_printf("Buffer '%s' doesn't exist.", buffname);
		return;
	}

	view_setbuffer(view_current, buffer);
}

void command_setopt() {
	u_char *option;
	u_char *value;
	opt_t *opt;

	option = minibuff_optprompt(NULL);
	if (!option)
		return;

	/* find the option */
	opt = options_lookup(option);
	if (!opt) {
		minibuff_printf("Nonexistant option '%s'.", option);
		goto free;
	}

	/* if it's not setable at runtime, inform the user */
	if (!opt->setable) {
		minibuff_printf("Option '%s' not runtime setable.", option);
		goto free;
	}

	/* just toggle booleans */
	if (opt->type == OPT_BOOL) {
		*((u_int *) opt->val) = !*((u_int *) opt->val);
		minibuff_printf("Toggled boolean option '%s' to %s.",
			option, *((u_int *) opt->val) ? "true" : "false");
		goto free;
	}

	/* other types need to ask for a value */
	value = minibuff_prompt("Value", NULL);
	if (!value)
		goto free;
	options_set(opt, value);

	free(value);
free:
	free(option);
}

void command_openfile() {
	buffer_t *buffer;
	u_char *fn, *exfn;
	u_char *dir;
	viewhdr_t *v, *stack;
	int all, cnt;

	v = view_current;

	/*
	 * prompt to save the buffer if this view contains the last
	 * references to the buffer
	 */
	all = 0;
	stack = v = view_current;
	cnt = v->buffer->refcnt;
	do {
		if (stack->buffer == v->buffer)
			cnt--;
		stack = stack->under;
	} while (stack);
	if (!cnt && bufsaveloop(v->buffer, &all, 0) == -1)
		return;

	/* ask for the filename */
	dir = dirpart(v->buffer->name);
	fn = minibuff_fileprompt(dir);
	if (dir)
		free(dir);
	if (!fn)
		return;

	/* change the view's buffer to the new file */
	exfn = tilde_expand(fn);
	buffer = file_read(exfn);
	if (!buffer) {
		if (errno == ENOENT)
			buffer = buffer_create(exfn, 0);
		else {
			minibuff_printf("Error reading file: %s",
				strerror(errno));
			goto free;
		}
	}
	view_setbuffer(v, buffer);

free:
	free(exfn);
	free(fn);
}

void command_save() {
	buffer_t *buffer;

	buffer = view_current->buffer;
	if (buffer_save(buffer))
		minibuff_printf("Error saving file: %s.", strerror(errno));
	else
		minibuff_printf("File saved.");
}

void command_saveas() {
	buffer_t *buffer;
	u_char *fn, *dir;

	buffer = view_current->buffer;

	dir = dirpart(buffer->name);
	fn = minibuff_fileprompt(dir);
	if (dir)
		free(dir);
	if (!fn)
		return;

	free(buffer->name);
	buffer->name = fn;
	buffer->dirty = 1;
	command_save(buffer);
}

void command_viewcycle() {
	if (TAILQ_FIRST(&frame_active->views))
		frame_focview(frame_active, TAILQ_FIRST(&frame_active->views));
}

/*
 * prompt the user for the name of a type, instantiate a new view of that
 * type and stack it over the current one.
 */
void command_pushview() {
	viewtype_t *type;
	u_char *s;

	/* get the viewtype name */
	s = minibuff_vtypeprompt(NULL);
	if (!s)
		return;
	if (!strcmp(s, "vdefault") || !strcmp(s, "vminibuff")) {
		minibuff_printf("Not allowed.");
		free(s);
		return;
	}
	type = view_findtype(s);
	free(s);

	/* make sure it's a valid type */
	if (!type) {
		minibuff_printf("Bogus view type.");
		return;
	}

	/* push the new view over it */
	view_push(&frame_active->views, type, view_current);
	view_current->buffer->dirty = 1;
	minibuff_printf("View pushed.");
}

/* pop the top view off of the view stack for view_current */
void command_popview() {
	if (!view_current->under) {
		minibuff_printf("Can't pop the bottom-most view.");
		return;
	}

	view_pop(&frame_active->views, view_current);
	view_current->buffer->dirty = 1;
	minibuff_printf("View popped.");
}

void command_vertsplit() {
	frame_vertsplit(frame_active, view_current);
}

void command_horizsplit() {
	frame_horizsplit(frame_active, view_current);
}

void command_rmview() {
	viewhdr_t *v, *stack;
	int cnt, all;

	all = 0;
	v = stack = view_current;
	cnt = v->buffer->refcnt;
	do {
		if (stack->buffer == v->buffer)
			cnt--;
		stack = stack->under;
	} while (stack);
	if (!cnt && bufsaveloop(v->buffer, &all, 0) == -1)
		return;

	if (frame_rmview(frame_active, v))
		minibuff_printf("Can't remove last view.");
}

void command_rmotherviews() {
	viewhdr_t *v, *before, *stack, *cur;
	int ret, all, cnt;

	/*
	 * prompt to save any buffers who are about to lose their last
	 * reference(s).  this is somewhat complicated because each
	 * view in the stack will have a buffer reference in it's header.
	 */
	all = 0;
	cur = view_current;
	TAILQ_FOREACH(v, &frame_active->views, v_list) {
		if (v == cur)
			continue;
		cnt = v->buffer->refcnt;
		before = v;
		do if (before->buffer == v->buffer) {
			stack = before;
			do {
				if (stack->buffer == v->buffer)
					cnt--;
				stack = stack->under;
			} while (stack);
		} while ((before = TAILQ_PREV(before, viewlist, v_list)));
		assert(cnt >= 0);
		if (!cnt && (ret = bufsaveloop(v->buffer, &all, 1)) != 0) {
			if (ret == -1)
				return;
			else
				break;
		}
	}

	if (frame_oneview(frame_active, view_current))
		minibuff_printf("Can't remove last view.");
}

void command_vertgrow() {
	if (frame_size(frame_active, view_current, FRAME_VSPLIT, 1))
		minibuff_printf("Can't grow vertically.");
}

void command_vertshrink() {
	if (frame_size(frame_active, view_current, FRAME_VSPLIT, -1))
		minibuff_printf("Can't shrink vertically.");
}

void command_horizgrow() {
	if (frame_size(frame_active, view_current, FRAME_HSPLIT, 1))
		minibuff_printf("Can't grow horizontally.");
}

void command_horizshrink() {
	if (frame_size(frame_active, view_current, FRAME_HSPLIT, -1))
		minibuff_printf("Can't shrink horizontally.");
}

void command_framecycle() {
	frame_activate(TAILQ_LAST(&frame_list, framelist));
	frame_forcedraw(frame_active);
}

void command_newframe() {
	frame_create(view_duplicate(view_current), view_current->buffer);
	minibuff_printf("Allocated new frame.");
}

void command_rmframe() {
	viewhdr_t *before, *stack, *v;
	frame_t *frame;
	int ret, cnt, all;

	frame = frame_active;

	/*
	 * save any buffers on this frame who's last references will
	 * go away when this frame is gone.
	 */
	all = 0;
	TAILQ_FOREACH(v, &frame->views, v_list) {
		cnt = v->buffer->refcnt;
		before = v;
		do if (before->buffer == v->buffer) {
			stack = before;
			do {
				if (stack->buffer == v->buffer)
					cnt--;
				stack = stack->under;
			} while (stack);
		} while ((before = TAILQ_PREV(before, viewlist, v_list)));
		assert(cnt >= 0);
		if (!cnt && (ret = bufsaveloop(v->buffer, &all, 1)) != 0) {
			if (ret == -1)
				return;
			else
				break;
		}
	}

	command_framecycle();
	if (frame_active == frame)
		minibuff_printf("Can't remove last frame.");
	else {
		minibuff_printf("Frame removed.");
		frame_destroy(frame);
	}
}
