/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef COMMAND_H
#define COMMAND_H

/* command types */
enum {
	COMMAND_NORMAL,
	COMMAND_FUNC
};

/* used for the command name to keysym map */
struct command {
	LIST_ENTRY(command) c_list;
	LIST_ENTRY(command) c_rev;

	int	type;
	u_char	*name;
	int	keysym;
	void	(*handler)(command_t *);	/* for COMMAND_FUNC */
};

LIST_HEAD(commandlist, command);

/* hash of names to commands */
#define COMMAND_BUCKETS		128
extern commandlist_t		*command_hash;

void		command_init();
void		command_shutdown();
void		command_add(u_char *, int keysym);
void		command_addf(u_char *, int keysym,
			void (*handler)(command_t *));
void		command_remove(u_char *);
int		command_getkeysym(u_char *);
command_t	*command_getcmd(int keysym);

/* some commands */
int	command_excmd();
void	command_hashstats();
int	command_savebuffers();
void	command_switchbuff();
void	command_setopt();
void	command_openfile();
void	command_save();
void	command_saveas();
void	command_viewcycle();
void	command_pushview();
void	command_popview();
void	command_vertsplit();
void	command_horizsplit();
void	command_rmview();
void	command_rmotherviews();
void	command_vertgrow();
void	command_vertshrink();
void	command_horizgrow();
void	command_horizshrink();
void	command_framecycle();
void	command_newframe();
void	command_rmframe();

#endif
