/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef EDITOR_H
#define EDITOR_H

#ifndef __GNUC__
# error This program will not compile without GNUC extensions.
#endif

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <assert.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <limits.h>
#include <locale.h>
#include <signal.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>

/* this isn't present on Cygwin */
#ifndef _POSIX_VDISABLE
# define _POSIX_VDISABLE 0xff
#endif

#ifdef HAVE_PATHS_H
# include <paths.h>
#else
# define _PATH_BSHELL	"/bin/sh"
#endif

#ifdef HAVE_GETOPT_H
# include <getopt.h>
#endif

#include "bsd_queue.h"
#include "bsd_vis.h"
#include "bsd_err.h"
#include "bsd_string.h"

#include "forward.h"

#include "ckmalloc.h"
#include "options.h"
#include "util.h"
#include "line.h"
#include "undo.h"
#include "buffer.h"
#include "screen.h"
#include "keybind.h"
#include "file.h"
#include "word.h"
#include "para.h"
#include "killring.h"
#include "minibuff.h"
#include "input.h"
#include "hash.h"
#include "search.h"
#include "macro.h"
#include "color.h"
#include "view.h"
#include "frame.h"
#include "region.h"
#include "vdefault.h"
#include "draw.h"
#include "command.h"
#include "rcfile.h"
#include "vminibuff.h"
#include "staticmod.h"
#include "complete.h"
#include "module.h"
#include "cmd_vdef/cmd_vdef.h"

#ifdef USE_DMALLOC
# include <dmalloc.h>
#endif

#define VERSION		"2.0"

extern char *argv0;

#endif
