/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

/* dummy structure used for creating hashtbls */
struct dummy {};

/* allocate a hashtable */
void *hash_alloc(u_char *name, int buckets, int off_next) {
	LIST_HEAD(, dummy) *hashtbl;
	int i;

	hashtbl = ckmalloc(sizeof(*hashtbl) * buckets);
	for (i = 0; i < buckets; i++)
		LIST_INIT(&hashtbl[i]);

#ifndef NDEBUG
	{
		hashtab_t *hashtab;

		/*
		 * add a hashtab list entry to make it easier to find
		 * hashtables to print statistic information about.
		 */
		hashtab = ckmalloc(sizeof(hashtab_t));
		hashtab->name = ckstrdup(name);
		hashtab->table = hashtbl;
		hashtab->buckets = buckets;
		hashtab->off_next = off_next;
		LIST_INSERT_HEAD(&hash_list, hashtab, ht_list);
	}
#endif

	return hashtbl;
}

#ifndef NDEBUG

/* list of hashtables used to report statistics */
hashtablist_t hash_list = LIST_HEAD_INITIALIZER(hash_list);

/* return a string with some hashtable statistic information */
u_char *hash_stats(void *table, int buckets, int off_next) {
	u_char string[2048];
	LIST_HEAD(, dummy) *hashtbl;
	float avg;
	char *item;
	u_int coll_max, coll_min, coll_tot;
	int idx, cnt, ents, effcy;

	hashtbl = table;

	/* determine collision statistics */
	coll_min = UINT_MAX;
	coll_tot = ents = coll_max = 0;
	for (idx = 0; idx < buckets; idx++) {
		cnt = 0;
		item = (char *) LIST_FIRST(&hashtbl[idx]);
		while (item != NULL) {
			cnt++, ents++;
			item = *((char **) (item + off_next));
		}
		/* remove first (non collided) entry */
		if (--cnt < 0)
			cnt = 0;
		if (cnt > coll_max)
			coll_max = cnt;
		if (cnt < coll_min)
			coll_min = cnt;
		coll_tot += cnt;
	}
	avg = (float) coll_tot / buckets;


	if (ents)
		effcy = ((float) (ents - coll_tot) / ents) * 100;
	else
		effcy = 100;

	snprintf(string, sizeof(string), "%2d%% efcy. %d buckets. %d "
		"ents. collisions (min,max,tot,avg): %d,%d,%d,%2.2f",
		effcy, buckets, ents, coll_min, coll_max, coll_tot, avg);
	return ckstrdup(string);
}

#endif
