/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef HASH_H
#define HASH_H

/* just normal free */
#define	hash_free	free

void	*hash_alloc(u_char *name, int buckets, int nextoffs);

/* hash a counted string */
static __inline int hash_mem(int buckets, u_char *s, int len) {
	u_int sum = 0;

	while (len--)
		sum += sum * 33 + *s++;
	return sum % buckets;
}

/* hash a string */
static __inline int hash_string(int buckets, u_char *s) {
	return hash_mem(buckets, s, strlen(s));
}

/*
 * stuff for debug; determining appropriate hashtable sizes, checking
 * hash algorithm efficency, and such.
 */
#ifndef NDEBUG

/*
 * this is used to track hashtables for the sole purpose of making
 * it easy to give stats for all hashtables in the system.
 */
struct hashtab {
	LIST_ENTRY(hashtab) ht_list;

	void	*table;
	u_char	*name;
	int	buckets;
	int	off_next;
};
LIST_HEAD(hashtablist, hashtab);
extern hashtablist_t hash_list;

u_char	*hash_stats(void *table, int buckets, int off_next);

#endif

#endif
