/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

/*
 * insert a character into a line at the specified location,
 * and resize the line if neccesary.
 */
void line_putc(line_t *line, int loc, u_char c) {
	assert(loc >= 0 && loc <= line->length);

	/* reallocate if neccesary */
	if (line->length + 1 >= line->size) {
		line->size *= 2;
		line->text = ckrealloc(line->text, line->size);
	}

	/* put the character into the line */
	if (line->length - loc)
		memmove(&line->text[loc + 1], &line->text[loc],
			line->length - loc);
	line->text[loc] = c;
	line->length++;

	return;
}

/*
 * insert a string into a line at the specified location,
 * and resize the line if neccesary.
 */
void line_nputs(line_t *line, int loc, u_char *s, int slen) {
	if (!slen)
		return;

	assert(loc >= 0 && loc <= line->length);

	/*
	 * can't be called with a s inside of text, because it messes
	 * up when we reallocate and the memmove can break stuff
	 */
	assert(!(s > line->text && s < line->text + line->size));

	/* reallocate if neccesary */
	if (line->length + slen >= line->size) {
		while (line->size < line->length + slen)
			line->size *= 2;
		line->text = ckrealloc(line->text, line->size);
	}

	/* copy the string into the middle of the line */
	if (line->length - loc)
		memmove(&line->text[loc + slen], &line->text[loc],
			line->length - loc);
	memcpy(&line->text[loc], s, slen);
	line->length += slen;

	return;
}
