/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "vfolding.h"

fold_t *fold_add(foldlist_t *list, int start, int end) {
	fold_t *fold, *before;

	fold = ckmalloc(sizeof(fold_t));
	fold->start = start;
	fold->end = end;
	TAILQ_INIT(&fold->subfolds);

	before = fold_findbefore(list, start);
	/*
	 * todo; merge subfolds
	 */
	if (before)
		TAILQ_INSERT_AFTER(list, before, fold, f_list);
	else
		TAILQ_INSERT_HEAD(list, fold, f_list);

	return fold;
}

/* find the first fold who's region ends after linenum */
fold_t *fold_findafter(foldlist_t *list, int linenum) {
	fold_t *fold;

	TAILQ_FOREACH(fold, list, f_list)
		if (fold->end >= linenum)
			return fold;
	return NULL;
}

/* find the last fold who's region ends before linenum */
fold_t *fold_findbefore(foldlist_t *list, int linenum) {
	fold_t *fold, *last;

	last = NULL;
	TAILQ_FOREACH(fold, list, f_list)
		if (fold->end < linenum)
			last = fold;
		else /* if (fold->end >= linenum) */
			break;
	return last;
}
