/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef FOLD_H
#define FOLD_H

typedef struct fold fold_t;
typedef struct foldlist foldlist_t;

/*
 * folds are represented as a tree of line number regions.  the
 * children of these regions are sorted according to linenumber, which
 * allows for reasonably simple determination of the next/last/etc
 * linenumber that enters (or exits) a fold.
 *
 * folds in a buffer a described by placed markers in the text on
 * lines which describe the starts and ends of folded regions.  (the
 * exact text of these markers is configurable, but it is usually
 * set to begin with '{{{', and end with '}}}').  only the regions
 * of the text that are actually folded are stored in this tree; there
 * is no need to track the state of unfolded regions.
 *
 * the tree structure allows for nesting of fold markers -- unfolded
 * nests aren't stored here (because, as said above, there's no need
 * to store information about open folds in this tree), but if a fold
 * is made in a region of text containing one (or more) other folds,
 * those child folds are placed in subfolds to allow their foldedness
 * to be maintained when the parent fold is opened.
 */
TAILQ_HEAD(foldlist, fold);
struct fold {
	TAILQ_ENTRY(fold) f_list;

	int		start, end;	/* folded line region */
	foldlist_t	subfolds;
};

fold_t	*fold_add(foldlist_t *, int start, int end);
fold_t	*fold_findafter(foldlist_t *, int);
fold_t	*fold_findbefore(foldlist_t *, int);

#endif
