/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "vfolding.h"

/* {{{ Input handling */
static int findmark(vfolding_t *v, line_t **line, int *linenum, int back) {
	u_char *c;
	int nest;

	if (!back) {
		*line = TAILQ_NEXT(*line, l_list);
		++*linenum;
	}

	nest = 0;
	for (;;) {
		c = strnstr((*line)->text, back ? vfolding_endmark :
			vfolding_begmark, (*line)->length);
		if (c)
			nest++;
		c = strnstr((*line)->text, back ? vfolding_begmark :
			vfolding_endmark, (*line)->length);
		if (c && nest-- == 0)
			return 0;

		if (back) {
			*line = TAILQ_PREV(*line, linelist, l_list);
			--*linenum;
		} else {
			*line = TAILQ_NEXT(*line, l_list);
			++*linenum;
		}
		if (*line == NULL)
			break;
	}

	return -1;
}

static void folddelim(vfolding_t *v, line_t **line, int *linenum) {
	line_t *eline, *sline;
	int elinenum, slinenum;

	sline = eline = *line;
	slinenum = elinenum = *linenum;
	if (findmark(v, &sline, &slinenum, 1) == -1)
		return;
	if (findmark(v, &eline, &elinenum, 0) == -1)
		return;
	assert(elinenum != slinenum);
	fold_add(&v->folds, slinenum, elinenum);

	*line = sline;
	*linenum = slinenum;
}

void vfolding_input(vfolding_t *v, int keysym) {
	if (keysym == KEYSYM_FOLD) {
		folddelim(v, &v->vdef->cursor.line, &v->vdef->cursor.linenum);
		v->vdef->redraw_text = 1;
		v->vdef->redraw_header = 1;
		return;
	}

	VIEWCALL(v->hdr.under, input, keysym);
}
/* }}} */
