/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef ANCHOR_H
#define ANCHOR_H

/*
 * to avoid parsing the whole buffer when changes are made, a
 * state value is saved at places in the buffer where changes
 * in state occur; this allows the vhighlight view to handle
 * notifications of changes in the buffer and decide from where
 * it is neccesary to parse when drawing.
 */
typedef struct anchor {
	TAILQ_ENTRY(anchor) a_list;

	synitem_t	*state;
	line_t		*line;
	int		linenum;
} anchor_t;

typedef TAILQ_HEAD(anchorlist, anchor) anchorlist_t;

void		anchor_shutdown();
anchor_t	*anchor_add(anchorlist_t *, line_t *line, int linenum,
			synitem_t *state);
void		anchor_remove(anchorlist_t *, anchor_t *);
anchor_t	*anchor_findlast(anchorlist_t *, int linenum);

/* clear out all anchors in a list */
static __inline void anchor_emptylist(anchorlist_t *anchors) {
	while (!TAILQ_EMPTY(anchors))
		anchor_remove(anchors, TAILQ_FIRST(anchors));
}

#endif
