/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef SYNTAX_H
#define SYNTAX_H

typedef struct synitem	synitem_t;
typedef struct syntax	syntax_t;

/* types of syntax items */
enum {
	SYNITEM_REGEXP,
	SYNITEM_WORDS,
	SYNITEM_TOKENS,
	SYNITEM_BLOCK,
	SYNITEM_TOEOL,
	SYNITEM_STRING
};

/*
 * a node in the syntax definition; describes a portion of the
 * language (i.e. a list of reserved words, or something that
 * enters a block state such as a comment, and so on) and what
 * color to highlight as.
 */
struct synitem {
	TAILQ_ENTRY(synitem) si_list;

	int	type;
	int	color;

	union {
		u_char	**wordlist;
		regex_t	*re;

		struct {
			u_char **begin;
			u_char **end;
		} blk;
		struct {
			u_char tok;
			u_char quote;
		} str;
	} u;
};
typedef TAILQ_HEAD(synitemlist, synitem) synitemlist_t;

/*
 * specification of how to perform syntax highlighting
 * on a particular language.
 */
struct syntax {
	LIST_ENTRY(syntax) s_list;

	u_char		*lang;
	synitemlist_t	items;

	/* syntax definition options */
	u_int	hlnums:1;	/* highlight constant ints/doubles, etc */
};
typedef LIST_HEAD(, syntax) syntaxlist_t;

void		syntax_init();
void		syntax_shutdown();
syntax_t	*syntax_findlang(u_char *);
int		syntax_highlight(syntax_t *syntax, line_t *,
			int *idx, synitem_t **state);

syntax_t	*syntax_load(u_char *);

#endif
