/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef VHIGHLIGHT_H
#define VHIGHLIGHT_H

#include "editor.h"
#include <regex.h>
#include "syntax.h"
#include "anchor.h"

/* where to look for syntax definitions */
#define SYNTAX_PATH	"~/.led/syntax:" DATADIR "/syntax"

/*
 * view that overrides drawing to provide language-based syntax
 * highlighting.
 */
typedef struct {
	viewhdr_t	hdr;

	/* the vdefault somewhere below us */
	vdefault_t	*vdef;

	/* highlighting state anchors */
	anchorlist_t	anchors;

	/* valid state position in the buffer */
	struct {
		line_t	*line;
		int	linenum;
	} valid;

	/*
	 * information about how to highlight the data contained in
	 * this buffer
	 */
	syntax_t	*syntax;
} vhighlight_t;

extern int vhighlight_numberclr;

int	vhighlight_init();
void	vhighlight_shutdown();

#endif
