/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef OPTIONS_H
#define OPTIONS_H

/* how long paren flashing flashes the matching paren */
#define FLASH_DURATION		300

/* types of options */
enum {
	OPT_INT,
	OPT_UINT,
	OPT_BOOL,
	OPT_CHAR,
	OPT_STRING,
	OPT_COLOR,
	OPT_CALL
};

/* entry in the options hash */
struct opt {
	LIST_ENTRY(opt) o_list;

	int	type;
	u_char	*name;
	int	setable;/* option is setable at runtime */

	/*
	 * pointer to either storage for the value; or in the case of
	 * OPT_CALL points to a function that is called with a single
	 * argument char * when the option is changed.
	 */
	void	*val;
};
LIST_HEAD(optlist, opt);

/* hash of option names to opt_t */
#define OPTIONS_BUCKETS	16
extern optlist_t	*options_hash;

/* general options are accessible here */
struct options {
	u_char	*autoviews;
	int	tabwidth;
	int	killringmax;
	int	markringmax;
	int	stickyhomekey;
	int	chunkyscroll;
	int	noexplicitmark;
	int	highbitmeta;
	int	icasesearch;
	int	autoindent;
	int	parenflash;
	int	forcefilenl;
	int	printnlatend;
	u_char	intrchar;
	u_char	blanklinechar;
};

extern options_t options;

void	options_init();
void	options_shutdown();
void	options_add(u_char *name, int type, void *var, int setable);
void	options_rm(u_char *);
opt_t	*options_lookup(u_char *name);
void	options_set(opt_t *, u_char *strval);

#endif
