/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

/*
 * paragraph location routines; paragraphs are defined by lines
 * containing only whitespace.
 */

void para_next(line_t **line, int *linenum) {
	int i;

	/* first pass all adjacent white-space lines */
	for (;;) {
		for (i = 0; i < (*line)->length; i++)
			if (!isspace((*line)->text[i]))
				goto para;

		if (!TAILQ_NEXT(*line, l_list))
			return;
		*line = TAILQ_NEXT(*line, l_list);
		(*linenum)++;
	}

para:
	/* look for the first blank line after that */
	for (;;) {
		for (i = 0; i < (*line)->length; i++)
			if (!isspace((*line)->text[i]))
				goto next;

		/* found the start of the next paragraph */
		return;

next:
		if (!TAILQ_NEXT(*line, l_list))
			return;
		*line = TAILQ_NEXT(*line, l_list);
		(*linenum)++;
	}
}

void para_prev(line_t **line, int *linenum) {
	int i;

	/* first pass adjacents */
	for (;;) {
		for (i = 0; i < (*line)->length; i++)
			if (!isspace((*line)->text[i]))
				goto para;

		if (!TAILQ_PREV(*line, linelist, l_list))
			return;
		*line = TAILQ_PREV(*line, linelist, l_list);
		(*linenum)--;
	}

para:
	/* look for the first blank line after that */
	for (;;) {
		for (i = 0; i < (*line)->length; i++)
			if (!isspace((*line)->text[i]))
				goto prev;

		/* found the start of the prev paragraph */
		return;

prev:
		if (!TAILQ_PREV(*line, linelist, l_list))
			return;
		*line = TAILQ_PREV(*line, linelist, l_list);
		(*linenum)--;
	}
}
