%{
/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"
#include "rcfile.tab.h"

/* translate \" quotes in a string from the rcfile */
u_char *rclex_makestr(u_char *yytext, int yyleng) {
	u_char *str;
	int len, i;

	/*
	 * make a copy of the string and chop the " chars.
	 * thie copy gets free()'d elsewhere. (yyparse, etc)
	 * switch all the \" into ", and \\ into \.
	 */
	str = ckstrdup(&yytext[1]);
	len = yyleng - 1;
	str[--len] = '\0';
	i = 0;
loop:
	for (; i < len; i++)
		if (str[i] == '\\') {
			if (++i >= len)
				break;
			/* extra left in len on memmoves to copy the null */
			switch (str[i]) {
			case '\n':
				memmove(&str[i - 1], &str[i + 1], len - i);
				len -= 2;
				goto loop;
			case '\"':
				str[i - 1] = '\"';
				memmove(&str[i], &str[i + 1], len - i);
				len--;
				goto loop;
			case '\\':
				str[i - 1] = '\\';
				memmove(&str[i], &str[i + 1], len - i);
				len--;
				goto loop;
			case 't':
				str[i - 1] = '\t';
				memmove(&str[i], &str[i + 1], len - i);
				len--;
				goto loop;
			}
		}

	return str;
}

%}

%option noyywrap
%option nounput
%option yylineno

PUNCTS		";"|"{"|"}"
STRING		\"([^\\\"]|\\.|\\n)*"\""

%%

"#".*\n		/* eat comments */

	/*
	 * data types
	 */

{STRING} {
	yylval.str = rclex_makestr(yytext, yyleng);
	return STRING;
}

"opt"			return OPT;
"key"			return KEY;
"keyclear"		return KEYCLEAR;
"cmdclear"		return CMDCLEAR;
"clearallkeys"		return CLEARALLKEYS;
"loadmod"		return LOADMOD;

{PUNCTS}	return yytext[0];
.|\n		/* eat */

%%
