/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef SCREEN_H
#define SCREEN_H

#include <curses.h>

#define KEYSYM_RESIZE		KEY_RESIZE
#define SCREEN_FIRSTSYM		KEY_MIN
#define SCREEN_FIRSTUSRSYM	(KEY_MAX + 1)

/* attribute definitions for colors */
#define SCREEN_A_NORM		A_NORMAL
#define SCREEN_A_STAND		A_STANDOUT
#define SCREEN_A_UL		A_UNDERLINE
#define SCREEN_A_RV		A_REVERSE
#define SCREEN_A_BLINK		A_BLINK
#define SCREEN_A_DIM		A_DIM
#define SCREEN_A_BOLD		A_BOLD

/* routines implemented as macros */
#define screen_reinit()		do { endwin(); refresh(); } while (0)
#define screen_clear		clear
#define screen_suspend		endwin
#define screen_redraw()		wrefresh(curscr)
#define screen_gotoxy(x, y)	move(y, x)
#define screen_width		COLS
#define screen_height		LINES
#define screen_putc(c)		addch(c)
#define screen_puts(s)		addstr(s)
#define screen_nputs(s, n)	addnstr(s, n)
#define screen_bell()		screen_needbell = 1

extern int screen_needbell;
extern int screen_keysyms;

void	screen_init();
void 	screen_shutdown();
void	screen_defcolor(int ident, u_char *fg, u_char *bg,
		int attrs, int bwattrs);
void	screen_setcolor(int ident);
void	screen_nokeysyms(int b);
int	screen_getkey();
int	screen_ungetkey(int ch);
int	screen_key_pending(int m);
int	screen_nprintf(int n, u_char *fmt, ...);
void	screen_refresh();

#endif
