/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "editor.h"

/* alphabetically sort an array of strings */
u_char **sortwords(u_char **words) {
	u_char *t;
	int i, n, j;

	/* do a simple sort into alphabetic order */
	for (i = 0; words[i]; i++)
		for (n = i + 1; words[n]; n++)
			for (j = 0; words[i][j]; j++) {
				if (words[n][j] == words[i][j])
					continue;
				if (words[n][j] < words[i][j])
					swap(t, words[i], words[n]);
				break;
			}

	return words;
}

/* expand ~'s in a string to the user's home directory */
u_char *tilde_expand(u_char *orig) {
	u_char *s, *home;
	int homelen, newlen, origlen;
	int i, n;

	home = getenv("HOME");
	homelen = strlen(home);
	origlen = strlen(orig);
	newlen = origlen;

	/* figure out how big to make the new allocation */
	for (i = 0; orig[i]; i++)
		if (orig[i] == '~' && orig[i + 1] == '/')
			newlen += homelen - 1;

	/* minor optimization */
	if (newlen == origlen)
		return ckstrdup(orig);

	/* allocate the new string and fill it out */
	s = ckmalloc(newlen + 1);
	for (i = n = 0; orig[i]; i++)
		if (orig[i] == '~' && orig[i + 1] == '/') {
			memcpy(&s[n], home, homelen);
			n += homelen;
		} else
			s[n++] = orig[i];
	s[n] = '\0';
	assert(n == newlen);

	return s;
}

/* return the directory part of a filename path */
u_char *dirpart(u_char *filename) {
	u_char *dir;
	u_char *slash;
	int len;

	slash = strrchr(filename, '/');
	if (!slash)
		return NULL;

	len = slash - filename + 1;
	dir = ckmalloc(len + 1);
	memcpy(dir, filename, len);
	dir[len] = '\0';

	return dir;
}

/* return the filename part of a directory path */
u_char *filepart(u_char *path) {
	u_char *slash;

	slash = strrchr(path, '/');
	if (!slash)
		return ckstrdup(path);
	return ckstrdup(&path[(slash - path) + 1]);
}
