/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef VDEFAULT_H
#define VDEFAULT_H

/*
 * the default view; most views stack on top of this.
 */
struct vdefault {
	viewhdr_t	hdr;

	/*
	 * locations in the buffer; the first three members of each
	 * of these structs are parallel so you can memcpy them.
	 */
	struct {
		line_t	*line;
		int	linenum;
		int	col;
	} view;

	struct {
		line_t	*line;
		int	linenum;
		int	col;
		int	screencol; 	/* col with ctl chars expanded */
		int	lastcol;	/* last screen col user moved to */
	} cursor;

	/*
	 * buffer location protection flags; view commands that remove lines
	 * or strings will set this flag before doing so if they want to
	 * prevent the rmstr and rmline handlers from doing certain things
	 * to either cursor, view or both.
	 */
#define PROTECT_CURSOR	(1 << 0L)
#define PROTECT_VIEW	(1 << 1L)
	int	rm_protect;

	/* flag if this vdefault is running a command */
	int	in_command;

	/*
	 * flag prevents paren flashing even if option is enabled;
	 * this is neccesary in situations where the view isn't the
	 * active view but commands are going to it in an unusual way,
	 * such as when a raw minibuffer routine reads a ) or }.
	 */
	int	inhibit_parenflash:1;

	/*
	 * redrawing flags; redraw_header gets set any time the
	 * cursor position changes.  redraw_text any time the
	 * range of visible text changes, or the text itself changes.
	 */
	int	redraw_header:1;
	int	redraw_text:1;
};

void	vdefault_init();
void	vdefault_getmarkrgn(vdefault_t *, region_t *);
void	vdefault_setscreencol(vdefault_t *);
void	vdefault_trylastcol(vdefault_t *);
void	vdefault_checkscroll(vdefault_t *);

#endif
