/*-
 * Copyright (c) 2002 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef VMINIBUFF_H
#define VMINIBUFF_H

/*
 * the minibuffer view
 */
struct vminibuff {
	viewhdr_t	hdr;

	/* state flag for the minibuffer */
	enum {
		MINIBUFF_NORMAL,
		MINIBUFF_NEEDREDRAW,
		MINIBUFF_DRAWMSG,
		MINIBUFF_PROMPT
	} state;

	/* used when prompting */
	u_char		*prompt;

	/*
	 * when promping a callback routine may be provided that
	 * can handle keys.  this allows for character-at-a-time
	 * style input prompting.  the function should return zero
	 * to cause normal processing of the keysym, or nonzero
	 * to indicate the key should not be further processed.
	 */
	int		(*inputcback)(vminibuff_t *, int keysym);
	void		(*postinputcback)(vminibuff_t *, int keysym);

	/*
	 * if the user is being prompted for data of certain types,
	 * a completion function will be supplied here that may be
	 * queried for possible matches to whatever the user has
	 * typed so far.
	 */
	u_char		**(*compfunc)(vminibuff_t *, int *startidx);

	/*
	 * popup window buffer and view; popup_view is used to determine
	 * if a popup is up (if it's not null).
	 */
	viewhdr_t	*popup_view;
	buffer_t	*popup_buffer;

	/*
	 * flag used when exiting from a prompt to indicate if
	 * the user hit the cancel button.
	 */
	int		canceled;
};

void	vminibuff_init();
void	vminibuff_hidesz(vminibuff_t *);
void	vminibuff_popup(vminibuff_t *, buffer_t *);
void	vminibuff_prepare(vminibuff_t *, u_char *prompt, u_char *def);
void	vminibuff_enter(vminibuff_t *);
void	vminibuff_tmpexit(vminibuff_t *);
void	vminibuff_exit(vminibuff_t *);

#endif
